@@ -369,7 +369,7 @@ from ae.console import ConsoleApp                   # type: ignore
 from ae.files import FilesRegister, RegisteredFile  # type: ignore
 
 
-__version__ = '0.0.21'
+__version__ = '0.0.22'
 
 
 AppStateType = Dict[str, Any]                   #: app state config variable type
@@ -400,9 +400,9 @@ FLOW_KEY_SEP = ':'
 FLOW_ACTION_RE = re.compile("[a-z0-9]+")
 FLOW_OBJECT_RE = re.compile("[A-Za-z0-9_]+")
 
-ACTIONS_EXTENDING_FLOW_PATH = ['enter', 'add', 'edit', 'open', 'confirm']
-ACTIONS_REDUCING_FLOW_PATH = ['leave', 'close']
-ACTIONS_CHANGING_FLOW_WITHOUT_CONFIRMATION = ['', 'enter', 'leave', 'focus']
+ACTIONS_EXTENDING_FLOW_PATH = ['add', 'confirm', 'edit', 'enter', 'open', 'show']
+ACTIONS_REDUCING_FLOW_PATH = ['close', 'leave']
+ACTIONS_CHANGING_FLOW_WITHOUT_CONFIRMATION = ['', 'enter', 'focus', 'leave']
 
 
 def id_of_flow(action: str, obj: str, key: str = "") -> str:
@@ -566,7 +566,7 @@ class MainAppBase(ConsoleApp, ABC):
         :param new_value:   new value of the app state to change.
         :param send_event:  pass False to prevent to send/call the on_<state_name> event to the main app instance.
         """
-        self.dpo(f"MainAppBase.change_app_state({state_name}, {new_value}, {send_event})"
+        self.vpo(f"MainAppBase.change_app_state({state_name}, {new_value}, {send_event})"
                  f" flow={self.flow_id} last={self._last_focus_flow_id} path={self.flow_path}")
         setattr(self, state_name, new_value)
         self._change_observable(state_name, new_value)
@@ -602,7 +602,7 @@ class MainAppBase(ConsoleApp, ABC):
         :return:                True for to confirm change of flow id.
         """
         font_size: float = event_kwargs['font_size']
-        self.dpo(f"MainAppBase.on_font_size_change {font_size}")
+        self.vpo(f"MainAppBase.on_font_size_change {font_size}")
         self.change_app_state('font_size', font_size)
         return True
 
@@ -614,7 +614,7 @@ class MainAppBase(ConsoleApp, ABC):
         :return:                True for to confirm change of flow id.
         """
         light_theme: bool = event_kwargs['light_theme']
-        self.dpo(f"MainAppBase.on_light_theme_change to {light_theme}")
+        self.vpo(f"MainAppBase.on_light_theme_change to {light_theme}")
         self.change_app_state('light_theme', light_theme)
         return True
 
@@ -639,8 +639,13 @@ class MainAppBase(ConsoleApp, ABC):
 
         app_state = self.retrieve_app_states()
         for key, state in app_state.items():
+            if key == 'flow_id' and flow_action(state) != 'focus':
+                state = id_of_flow('', '')
+            elif key == 'flow_path':
+                state = [flow_id for flow_id in state if flow_action(flow_id) == 'enter']
+
             err_msg = self.set_var(key, state, section=APP_STATE_SECTION_NAME)
-            self.dpo(f"MainAppBase.save_app_state {key}={state} {err_msg or 'OK'}")
+            self.vpo(f"MainAppBase.save_app_state {key}={state} {err_msg or 'OK'}")
             if err_msg:
                 break
         self.load_cfg_files()
@@ -654,7 +659,7 @@ class MainAppBase(ConsoleApp, ABC):
 
         :param app_state:   dict of app states.
         """
-        self.dpo(f"MainAppBase.setup_app_states {app_state}")
+        self.vpo(f"MainAppBase.setup_app_states {app_state}")
         for key, val in app_state.items():
             self.change_app_state(key, val, send_event=False)
             if key == 'flow_id' and flow_action(val) == 'focus':
@@ -673,7 +678,7 @@ class MainAppBase(ConsoleApp, ABC):
                 upd_app_state('sound_volume', 0.96)
             elif version == 2:
                 if 'context_id' not in app_state:
-                    continue    # update only LiszApp apps
+                    continue    # update - only needed for LiszApp apps
 
                 def mig_node(parent_node):
                     """ recursively migrate sub_list to node key. """
@@ -687,8 +692,8 @@ class MainAppBase(ConsoleApp, ABC):
                 upd_app_state('root_node', mig_node(app_state['data_tree']), 'data_tree')
                 upd_app_state('flow_path_ink', app_state['context_path_ink'], 'context_path_ink')
                 upd_app_state('flow_id_ink', app_state['context_id_ink'], 'context_id_ink')
-                upd_app_state('flow_path', app_state['context_path'], 'context_path')
-                upd_app_state('flow_id', app_state['context_id'], 'context_id')
+                upd_app_state('flow_path', [], 'context_path')
+                upd_app_state('flow_id', '', 'context_id')
 
         if config_file_version < APP_STATE_CURRENT_VERSION:
             upd_app_state(APP_STATE_VERSION_VAR_NAME, APP_STATE_CURRENT_VERSION)
@@ -744,7 +749,7 @@ class MainAppBase(ConsoleApp, ABC):
         if not self.call_method(flow_event_name(ifi), ifi, event_kwargs) \
                 and not self.on_flow_change(ifi, event_kwargs) \
                 and action not in ACTIONS_CHANGING_FLOW_WITHOUT_CONFIRMATION:
-            self.dpo(f"MainAppBase.change_flow REJECTED '{ifi}'; flow='{self.flow_id}' path={self.flow_path}")
+            self.vpo(f"MainAppBase.change_flow REJECTED '{ifi}'; flow='{self.flow_id}' path={self.flow_path}")
             return False
 
         has_flow_focus = flow_action(self.flow_id) == 'focus'
@@ -753,6 +758,11 @@ class MainAppBase(ConsoleApp, ABC):
             self.change_app_state('flow_path', self.flow_path)
             flow_id = id_of_flow('', '') if action == 'enter' else ifi
         elif action in ACTIONS_REDUCING_FLOW_PATH:
+            # dismiss gets sent sometimes twice (e.g. on heavy double-clicking on drop-down-open-buttons)
+            # .. therefore prevent run-time error
+            if not self.flow_path:
+                self.dpo(f"MainAppBase.change_flow: ignoring multiple close action '{action}'")
+                return False
             ended_flow_id = self.flow_path.pop()
             self.change_app_state('flow_path', self.flow_path)
             if action == 'leave':
@@ -775,7 +785,7 @@ class MainAppBase(ConsoleApp, ABC):
             self.call_method('on_flow_widget_focused')
             self._last_focus_flow_id = flow_id
 
-        self.dpo(f"MainAppBase.change_flow CHANGED '{ifi}'; flow='{self.flow_id}' last='{self._last_focus_flow_id}'"
+        self.vpo(f"MainAppBase.change_flow CHANGED '{ifi}'; flow='{self.flow_id}' last='{self._last_focus_flow_id}'"
                  f" path={self.flow_path}")
         return True
 
@@ -796,7 +806,7 @@ class MainAppBase(ConsoleApp, ABC):
         This method is mainly used as the last fallback clicked event handler of a FlowButton.
         """
         class_name = flow_popup_class_name(flow_id)
-        self.dpo(f"MainAppBase.on_flow_change '{flow_id}' {event_kwargs} lookup={class_name}")
+        self.vpo(f"MainAppBase.on_flow_change '{flow_id}' {event_kwargs} lookup={class_name}")
 
         if flow_id:
             popup_class = self.class_by_name(class_name)
@@ -862,7 +872,7 @@ class MainAppBase(ConsoleApp, ABC):
 
     def key_press_from_framework(self, modifiers: str, key: str) -> bool:
         """ dispatch key press event, coming normalized from the UI framework. """
-        self.dpo(f"MainAppBase.key_press_from_framework({modifiers}+{key})")
+        self.vpo(f"MainAppBase.key_press_from_framework({modifiers}+{key})")
         event_name = f'on_key_press_of_{modifiers}_{key}'
         en_lower = event_name.lower()
         if not self.call_method(event_name):
@@ -968,6 +978,6 @@ class MainAppBase(ConsoleApp, ABC):
         :param win_pos_size:    window geometry/coordinates: x, y, width, height.
         """
         self.framework_app.landscape = win_pos_size[2] >= win_pos_size[3]
-        self.dpo(f"MainAppBase.win_pos_size_change landscape={self.framework_app.landscape}, {win_pos_size}")
+        self.vpo(f"MainAppBase.win_pos_size_change landscape={self.framework_app.landscape}, {win_pos_size}")
         self.change_app_state('win_rectangle', win_pos_size)
         self.call_method('on_win_pos_size')
